# -*- python -*-
# $RCSfile: reporter_GUI.py,v $
# $Revision: 1.53 $
# $Author: reida $
# $Date: 2006/01/19 21:54:54 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



# GUI part of error reporting machine.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import excepthook
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import thread_enable
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import reporter
from oof2.common.IO import reportermenu
from oof2.common.IO.GUI import fileselector
from oof2.common.IO.GUI import gfxmenu
from oof2.common.IO.GUI import oof_mainiteration
from oof2.common.IO.GUI import subWindow
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO.GUI import mainmenuGUI
import gtk
import pango
import Queue
import string
import sys
import traceback
import os
import time

OOF = mainmenu.OOF

# Flag for indicating if we should auto_open. 
_message_window_auto_open = True

_allMessageWindows = {}

class MessageWindow(subWindow.SubWindow):
    count = 1
    def __init__(self):
        debug.mainthreadTest()
        self.menu_name = "MessageWindow_%d" % MessageWindow.count
        self.title = "OOF2 Messages %d" % MessageWindow.count
        self.windows_menu_name = "Message_%d" % MessageWindow.count
        
        subWindow.SubWindow.__init__(
            self, title=self.title, menu=self.menu_name)

        # We are locally responsible for the windows submenu items.
        self.gtk.connect("destroy", self.destroy)

        # raise_window function is provided by the SubWindow class.
        OOF.Windows.Messages.addItem(
            oofmenu.OOFMenuItem(
            self.windows_menu_name,
            help="Raise Message window %d." % MessageWindow.count,
            cli_only=0, no_log=1,
            gui_callback=self.raise_window) )
            
        MessageWindow.count += 1

        _allMessageWindows[self] = 1

        # Control box, with buttons.  These could be menu items.
        controlbox = gtk.Frame()
        controlbox.set_shadow_type(gtk.SHADOW_OUT)
        self.mainbox.pack_start(controlbox, expand=0, fill=0)
        controlinnards = gtk.VBox()
        controlbox.add(controlinnards)
        buttonbox = gtk.HBox()
        controlinnards.pack_start(buttonbox, expand=0, padding=2)
        savebutton = gtk.Button("Save...")
        guitop.top().tooltip(savebutton, "Save the contents of this window to a file.")
        buttonbox.pack_start(savebutton, expand=0, fill=0, padding=2)
        savebutton.connect('clicked', self.saveButtonCB)

        self.button_dict = {}
        self.signal_dict = {}
        for m in reporter.messageclasses:
            button = gtk.CheckButton(m)
            buttonbox.pack_end(button, fill=0, expand=0, padding=2)
            self.signal_dict[m] = button.connect("clicked", self.button_click)
            self.button_dict[m] = button
            guitop.top().tooltip(button, "Show or hide "+
                                 reporter.messagedescriptions[m])

        messagepane = gtk.ScrolledWindow()
        messagepane.set_border_width(4)
        messagepane.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.mainbox.add(messagepane)

        self.messages = gtk.TextView()
        self.messages.set_editable(False)
        self.messages.set_cursor_visible(False)
        self.messages.set_wrap_mode(gtk.WRAP_WORD)
        self.changeFont(mainmenuGUI.getFixedFont())

        self.gtk.set_default_size(
            90*gtkutils.widgetCharSize(self.messages), 200)

        # Get two marks to be used for automatic scrolling
        bfr = self.messages.get_buffer()
        enditer = bfr.get_end_iter()
        self.endmark = bfr.create_mark(None, enditer, False)
        self.midmark = bfr.create_mark(None, enditer, False)

        messagepane.add(self.messages)

        self.state_dict = {}
        for m in reporter.messageclasses:
            self.state_dict[m]=1
            
        self.sbcbs = [
            switchboard.requestCallback("write message", self.write_message),
            switchboard.requestCallback("change fixed font", self.changeFont)
            ]

        self.draw()

    def destroy(self, gtkobj):
        global _message_window_auto_open
        del _allMessageWindows[self]
        _message_window_auto_open = False
        map(switchboard.removeCallback, self.sbcbs)
        OOF.Windows.Messages.removeItem(self.windows_menu_name)

    def changeFont(self, fontname):
        font_desc = pango.FontDescription(fontname)
        if font_desc:
            self.messages.modify_font(font_desc)

    def draw(self):
        self.gtk.show_all()
        for (m,v) in self.state_dict.items():
            b = self.button_dict[m]
            b.handler_block(self.signal_dict[m])
            if v:
                b.set_active(1)
            else:
                b.set_active(0)
            b.handler_unblock(self.signal_dict[m])
        self.refresh()

    def refresh(self):
        debug.mainthreadTest()
        msgs = []
        self.messages.move_mark_onscreen(self.midmark)
        for m in reporter.messagemanager.all_messages():
            if self.state_dict[m[1]]:
                msgs.append(m[0])
        self.messages.get_buffer().set_text("\n".join(msgs)+"\n")
        self.messages.scroll_mark_onscreen(self.midmark)

    # After each button click, set your local state.
    def button_click(self, gtkobj):
        for (m,b) in self.button_dict.items():
            if b.get_active():
                self.state_dict[m]=1
            else:
                self.state_dict[m]=0
        self.refresh()

    def write_message(self, message_tuple):
        debug.mainthreadTest()
        if self.state_dict[message_tuple[1]]:
            bfr = self.messages.get_buffer()
##            bfr.insert(bfr.get_end_iter(), message_tuple[0]+"\n")
            bfr.insert(bfr.get_end_iter(), message_tuple[0]+"\n")
            self.messages.scroll_mark_onscreen(self.endmark)

    def saveButtonCB(self, button):
        file = fileselector.fileSelector(ident='SaveMessages',
                                         title='Save Messages')
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                # The arguments for the Save command are constructed
                # from the same list of names as the state_dict, so we
                # just have to pass the dict here.
                OOF.File.Save.Messages.callWithDefaults(filename=file,
                                                        mode=mode,
                                                        **self.state_dict)
                                          
            
# Turn off the command-line outputs of the message manager.
reporter.messagemanager.suppress_stdout()

##for m in reporter.messageclasses:
##    reporter.messagemanager.set_flag(m, None)

#defaultfont = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")


class WarningPopUp:
    # Argument "message" is a single string for the second GtkLabel.
    OK = 1
    NO_MORE = 2
    def __init__(self, message):
        debug.mainthreadTest()
        self.gtk = gtk.Dialog()
        self.gtk.set_title("OOF2 Warning")
##        self.gtk.set_usize(250,-1)

        self.gtk.vbox.set_spacing(3)

        self.gtk.vbox.pack_start(gtk.Label("WARNING"))
        self.gtk.vbox.pack_start(gtk.Label(message))

        self.gtk.add_button("OK", self.OK)
        disable_button = self.gtk.add_button("Disable warnings", self.NO_MORE)
        guitop.top().tooltip(disable_button,
                             "Warnings can be re-enabled in the Help menu of the main window.")

        self.gtk.vbox.show_all()
    def run(self):
        debug.mainthreadTest()
        return self.gtk.run()
    def close(self):
        debug.mainthreadTest()
        self.gtk.destroy()

def _warning_pop_up(message):
    if reporter.messagemanager.get_warning_pop_up():
        popup = WarningPopUp(message)
        result = popup.run()
        popup.close()
        if result == WarningPopUp.NO_MORE:
            ## This used to call OOF.Help.Popup_warnings(0), but menu
            ## items shouldn't call other menu items, and the warning
            ## was presumably triggered by a menu item, so it can't
            ## call OOF.Help.etc here.
            reporter.messagemanager.set_warning_pop_up(False)

switchboard.requestCallback("messagemanager warning", _warning_pop_up)


# The error pop up optionally allows you to view the traceback,
# and save it to a file.  The file is automatically named.
class ErrorPopUp:
    OK = 1
    ABORT = 2
    def __init__(self, type, value, tbacklist):
        debug.mainthreadTest()
##        global defaultfont
##        self.font = defaultfont
        self.tb_strings = []

        self.answer = None
        
        self.datestampstring = time.strftime("%Y %b %d %H:%M:%S %Z")
        self.gtk = gtk.Dialog()
        self.gtk.set_title("OOF2 Error")
##        self.gtk.set_usize(500, -1)

        self.gtk.vbox.set_spacing(3)
        classname = string.split(str(type),'.')[-1]
        self.error_string = traceback.format_exception_only(type, value)
        self.gtk.vbox.pack_start(gtk.Label("ERROR"), expand=0, fill=0)

        frame = gtk.Frame()
        frame.set_border_width(6)
        frame.set_shadow_type(gtk.SHADOW_IN)
        self.gtk.vbox.pack_start(frame, expand=1, fill=1)

        fd = pango.FontDescription(mainmenuGUI.getFixedFont())

        errscroll = gtk.ScrolledWindow()
        errscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        frame.add(errscroll)
        self.errbox = gtk.TextView()    # error text goes here
        errscroll.add(self.errbox)
        self.errbox.set_editable(0)
        self.errbox.set_wrap_mode(gtk.WRAP_WORD)
        self.errbox.get_buffer().set_text("\n".join(self.error_string))
        self.errbox.modify_font(fd)

        self.gtk.add_button("OK", self.OK)
        self.gtk.add_button("Abort", self.ABORT)
        self.gtk.set_default_response(self.OK)

        self.tracebutton = gtk.Button("View Traceback")
        self.tracebutton.connect("clicked", self.trace)
        self.gtk.action_area.add(self.tracebutton)

        self.savebutton = gtk.Button("Save Traceback")
        self.savebutton.connect("clicked", self.savetrace)
        self.gtk.action_area.add(self.savebutton)

        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_border_width(3)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        
        self.tracepane = gtk.TextView()
        self.tracepane.set_editable(0)
        self.tracepane.set_wrap_mode(gtk.WRAP_WORD)
        self.tracepane.modify_font(fd)

        self.frame = gtk.Frame()
        self.frame.set_shadow_type(gtk.SHADOW_NONE)
        self.gtk.vbox.pack_start(self.frame)

        # Scroll not added to the frame until traceback button is pressed.
        self.scroll.add(self.tracepane)

        tblist = traceback.format_list(tbacklist)
        if tblist:
            self.tracepane.get_buffer().set_text("\n".join(tblist))
        else:
            self.savebutton.set_sensitive(0)
            self.tracebutton.set_sensitive(0)

        self.gtk.show_all()

    def close(self):
        self.gtk.destroy()

    def run(self):
        result = self.gtk.run()
        if result == self.ABORT:
            return self.ABORT
        return self.OK
            
    def trace(self, gtk):
        debug.mainthreadTest()
        c = self.frame.get_children()
        if len(c)==0:
            self.frame.add(self.scroll)
            self.tracebutton.set_label("Hide Traceback")
        else:
            self.frame.remove(c[0])
            self.tracebutton.set_label("View Traceback")
        self.gtk.show_all()

    def savetrace(self, gtk):
        fname = "traceback_oof."+str(os.getpid())
        errbuf = self.errbox.get_buffer()
        try:
            fobj = file(fname, "a+")
            fobj.write("\n%s\n" % self.datestampstring)
            fobj.writelines(self.tb_strings)
            fobj.writelines(self.error_string)
            #
            errbuf.insert(errbuf.get_end_iter(), "\nTraceback written to %s.\n"
                          % fname)
        except IOError:
            errbuf.insert(erfbuf.get_end_iter(),
                          "\nUnable to write traceback.\n")

# Blocking function, which raises an error pop up, blocks, and returns
# ErrorPopUp.OK if the "OK" button was pressed, and ErrorPopUp.ABORT
# otherwise.

def errorpopup_(e_type, e_value, tbacklist):
    e = ErrorPopUp(e_type, e_value, tbacklist)
    result = e.run()
    e.close()
    return result

# This gets reassigned at import time, but other imports can cause
# errors, so it needs to check if the mainloop is running.
def _gui_printTraceBack(e_type, e_value, tbacklist):
    if guitop.mainLoop():
        # Transfer control to the main thread to report errors in the GUI.
        res = mainthread.runBlock(errorpopup_, (e_type, e_value, tbacklist))
        if res == ErrorPopUp.ABORT:
            from oof2.common.IO.GUI import quit
            if mainthread.runBlock(quit.quitQuery):
                os.abort()
    else:                               # main loop not running!
        excepthook._printTraceBack(e_type, e_value, tbacklist)

excepthook.printTraceBack = _gui_printTraceBack


# Override the nonGUI function called by the message-window-creating
# menu callback.
##def newMessageWindow():
##    return MessageWindow()
##reporter._new_messages = newMessageWindow
reporter._new_messages = MessageWindow

# Module-level switchboard callback for auto-opening.  Takes an
# argument just because the signal we're using has one.
def _auto_open(message_tuple):
    global _message_window_auto_open
    if _message_window_auto_open and len(_allMessageWindows)==0:
        MessageWindow()
#        OOF.Windows.Messages.New()

switchboard.requestCallback("write message", _auto_open)

############################

def _saveMessages_GUI(menuitem):
    file = fileselector.fileSelector(
        ident='SaveMessages',
        title='Save Messages',
        params=[menuitem.get_arg(name) for name in reporter.messageclasses])
    if file is not None:
        mode = fileselector.get_file_mode(file)
        if mode is not None:
            menuitem.callWithDefaults(filename=file, mode=mode)

OOF.File.Save.Messages.add_gui_callback(_saveMessages_GUI)
