// -*- C++ -*-
// $RCSfile: progressbar.h,v $
// $Revision: 1.21 $
// $Author: langer $
// $Date: 2005/12/13 16:45:20 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

/* 
   In this file you find the building blocks to control threads.
   
   ThreadControl is meant to store the start/stop/continue variables 
   in the threaded process.

   Progress is the basic object to tracking the progress of a thread. 
   

*/

#ifndef PROGRESSBAR_H
#define PROGRESSBAR_H

#include <Python.h>
#include <string>
#include <pthread.h>
#include <sys/types.h>
#include <unistd.h>
#include "common/lock.h"

class ThreadState;
class ThreadID;

enum ProgressBarStatus {INCOMPLETE, COMPLETE, FAILED};


// C++ interface for the ProgressBar

class Progress {
private:
   Lock lockobj; // For locking when reading and writing to member
		 // variables of the progress bar
    
  float progress;/* 
		    Stores the fraction of closeness to the completion
		    of the running process/thread.
		    0: nothing done.
		    1: progress completed.
		 */


  int stop; // if set to not zero (by a click or CTRL-C) when queried
	    // the thread should be forced to stop and join.

  ProgressBarStatus success; 
  
  std::string bar_msg; // for graphics mode
  
  ThreadState * tState; // pointer to the current threadstate
  
  inline void lock() { lockobj.acquire(); }
  inline void unlock() { lockobj.release(); }
  void remove_from_list();
  Progress(const Progress&);	// prohibited!
  Progress();			// only called by getProgress().
public:
  ~Progress();
  //thread state controls
  void reset();
  inline int query_stop() { return stop; }
  void stop_it();
  void set_success();
  void set_failure();
  ProgressBarStatus get_success() const;

  ThreadState * get_threadstate() const { return tState; }
  ThreadID & get_thread_ID() const;

  //general thread variables update
  float get_progress(); //{return progress;};
  inline void set_progress(float var) { progress = var; }
  //gives or sets completion of process fraction with normalized 
  //values ranging from 0 to 1.

  //text messaging. Common to both text and graphics progress bars
  void set_message(const std::string &msg);
  const std::string get_message();
  void cleanup();
  void make_defunct();
  
  friend Progress *getProgress();
};


bool operator==(Progress &p1, Progress &p2);

Progress * getProgress();

#endif // PROGRESSBAR_H


