# -*- python -*-
# $RCSfile: meshdataGUI.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2005/08/08 21:39:34 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

## Window for displaying mesh data.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common import utils
from oof2.common.IO import gfxmanager
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import quit
from oof2.common.IO.GUI import subWindow
from oof2.common.IO.GUI import whowidget
from oof2.common.IO.GUI import widgetscope
from oof2.engine import mesh
from oof2.engine.IO.GUI import outputvalwidgets
import gtk
import oof2.common.IO.output

allMeshDataWindows = []

mdWindowMenu = mainmenu.OOF.Windows.addItem(oofmenu.OOFMenuItem("Mesh_Data"))
##mdWindowMenu.disable()

## TODO: Should adding the first Mesh layer to a window activate
## preexisting data viewers?  Or should they continue to say "No
## mesh"?

## TODO: Mesh Viewer widgets should update when new Fields are
## defined.  This may be an OutputParameterWidget problem.

## TODO: When a new Mesh is displayed in the Graphics Window, the data
## viewer should switch to it.  It should also do something sensible
## when a Mesh is deleted.

## TODO: The mesh viewer currently gives misleading results when a
## click occurs outside the boundary of the mesh -- the MeshInfo
## toolbox shows the true position of the click, but the data viewer
## gives the data at the "effective" position, which seems to be
## clamped to the nearest element boundary.

## TODO: This isn't really a meshviewer problem, but it's also true
## that the node or element is drawn at the undisplaced position,
## which can be disturbing if the mesh displayed is at the actual or
## enhanced positions.  

class MeshDataGUI(widgetscope.WidgetScope):
    count = 1
    def __init__(self, gfxwindow, position, output=None):
        mainthread.runBlock(self.__init__thread, (gfxwindow, position, output))

    def __init__thread(self, gfxwindow, position, output):
        allMeshDataWindows.append(self)
        widgetscope.WidgetScope.__init__(self, None)

        current_count = MeshDataGUI.count
        MeshDataGUI.count += 1
        self._name = "Mesh_Data_%d" % current_count
        self.output = output
        self.position = position
        self.sbcallbacks = []
        self.gsbcallbacks = []          # callbacks from a specific gfx window

        self.outputparam = oof2.common.IO.output.AggregateOutputParameter('output')

        # Although it's not displayed, we need a mesh widget in the
        # widgetscope, or the OutputParameterWidget won't work.
        # TODO LATER: Is this ugly, or what?
        self.meshwidget = whowidget.WhoWidget(mesh.meshes, scope=self)

        self.gtk = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.gtk.set_title(utils.underscore2space(self._name))
##        self.gtk.set_policy(1,1,1)
        self.mainbox = gtk.VBox()
        self.gtk.add(self.mainbox)

        # Put this window into the Windows menu.  The menu item can't
        # be logged, since the creation and operation of the window
        # aren't logged, so scripts shouldn't refer to it at all.
        mainmenu.OOF.Windows.Mesh_Data.addItem(oofmenu.OOFMenuItem(
            self._name,
            no_log=1,
            help="Raise Mesh Data window %d." % current_count,
            callback=self.raiseWindow))
        
        self.table = gtk.Table(rows=5, columns=3)
        self.mainbox.pack_start(self.table, expand=0, fill=0)

        label = gtk.Label("Source Window:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 0,1, xpadding=3, xoptions=gtk.FILL)
        guitop.top().tooltip(
            eventbox,
            "Display data for mouse clicks in this Graphics window.")

        self.gfxWindowChooser = chooser.ChooserWidget([],
                                                      callback=self.chooserCB)
        self.table.attach(self.gfxWindowChooser.gtk, 1,2, 0,1,
                     xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)

        self.freezeflag = 0
        self.freezebutton = gtk.CheckButton('Freeze')
        self.table.attach(self.freezebutton, 2,3, 0,1,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        self.freezebutton.set_active(self.freezeflag)
        self.freezebutton.connect('clicked', self.freezeButtonCB)
        guitop.top().tooltip(self.freezebutton,
                        "Prevent the data in this window from being updated.")

        label = gtk.Label("Mesh:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 1,2,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        guitop.top().tooltip(
            eventbox,
            "Data is displayed for values on this mesh.")

        self.meshtext = gtk.Entry()
        self.meshtext.set_editable(0)
        self.meshtext.set_size_request(12*guitop.top().charsize, -1)
        self.table.attach(self.meshtext, 1,3, 1,2,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)

        label = gtk.Label("Mouse x:")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 2,3,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        self.xtext = gtk.Entry()
        self.xtext.set_size_request(12*guitop.top().digitsize, -1)
        self.table.attach(self.xtext, 1,3, 2,3,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        self.xtext.set_editable(0)
        label = gtk.Label("Mouse y:")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 3,4,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        self.ytext = gtk.Entry()
        self.ytext.set_size_request(12*guitop.top().digitsize, -1)
        self.table.attach(self.ytext, 1,3, 3,4,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        self.ytext.set_editable(0)

        label = gtk.Label("Output:")
        label.set_alignment(1.0, 0.5)
        eventbox = gtk.EventBox()
        eventbox.add(label)
        self.table.attach(eventbox, 0,1, 4,5,
                          xpadding=3, xoptions=gtk.FILL, yoptions=0)
        guitop.top().tooltip(eventbox, "Choose which data is displayed.")
        
        self.outputwidget = self.outputparam.makeWidget(scope=self)
        self.table.attach(self.outputwidget.gtk, 1,3, 4,5,
                          xpadding=3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)
        switchboard.requestCallback(self.outputwidget, self.outputwidgetCB)

        hbox = gtk.HBox()
        self.mainbox.pack_start(hbox, expand=1, fill=1, padding=5)
        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_IN)
        hbox.pack_start(frame, expand=1, fill=1, padding=5)
        vbox = gtk.VBox()
        frame.add(vbox)
        self.databox = gtk.HBox()
        vbox.pack_start(self.databox, expand=1, fill=1, padding=3)
        self.datawidget = None       # set by updateData

        # Close button
        align = gtk.Alignment(xalign=0.5)
        self.mainbox.pack_start(align, expand=0, fill=0, padding=3)
        centerbox = gtk.HBox()
        align.add(centerbox)
        self.gtk.connect('destroy', self.destroyCB)
        self.closeButton = gtk.Button("Close")
        self.closeButton.connect('clicked', self.closeButtonCB)
        centerbox.add(self.closeButton)

        self.updateGfxWindowChooser()
        self.gfxWindowChooser.set_state(gfxwindow.name)
        if position is not None:
            self.updatePosition(position)
        self.currentMesh = None
        self.updateMesh()

        self.setupSwitchboard()         # gfx window dependent callbacks
        self.sbcallbacks += [
            switchboard.requestCallback('open graphics window',
                                        self.gfxwindowChanged),
            switchboard.requestCallback('close graphics window',
                                        self.gfxwindowChanged)
            ]

        
        self.gtk.show_all()

    def raiseWindow(self, menuitem):
        self.gtk.get_window()._raise()

    ###################

    # Gfx window management

    def gfxwindowChanged(self, window): # sb callback, window opened or closed
        self.updateGfxWindowChooser()

    def updateGfxWindowChooser(self):
        self.gfxWindowChooser.update(
            [w.name for w in gfxmanager.gfxManager.windows])

    def chooserCB(self, chsr, gfxwindowname):
        self.setupSwitchboard()
        self.updateMesh()               # calls updateData if needed

    def setupSwitchboard(self):
        map(switchboard.removeCallback, self.gsbcallbacks)
        window = self.currentGfxWindow()
        if window is not None:
            self.gsbcallbacks = [
                switchboard.requestCallback((window, "layers changed"),
                                            self.updateData),
                switchboard.requestCallback((window, "meshinfo click"),
                                            self.updatePosition)
            ]
        else:
            self.gsbcallbacks = []

    def currentGfxWindow(self):
        name = self.gfxWindowChooser.get_value()
        if name:
            return gfxmanager.gfxManager.getWindow(name)

    ##############

    def closeButtonCB(self, button):
        self.gtk.destroy()

    def destroyCB(self, *args):         # gtk callback
        mainmenu.OOF.Windows.Mesh_Data.removeItem(self._name)
##        if not mainmenu.OOF.Windows.Mesh_Data.items:
##            mainmenu.OOF.Windows.Mesh_Data.disable()
        map(switchboard.removeCallback, self.sbcallbacks)
        map(switchboard.removeCallback, self.gsbcallbacks)
        allMeshDataWindows.remove(self)
##        if not allMeshDataWindows:
##            mdWindowMenu.disable()

    ##############

    def freezeButtonCB(self, button):
        self.freezeflag = button.get_active()
        self.updateMesh()
        self.updatePosition(self.position)
        self.updateData()

    ##############

    def outputwidgetCB(self, interactive): # chosen output has changed
        self.updateData()

    def updateMesh(self):
        debug.mainthreadTest()
        if not self.freezeflag:
            window = self.currentGfxWindow()
            newmesh = None
            if window:
                newmesh = window.topwho('Mesh')
                if newmesh:
                    self.meshtext.set_text(newmesh.path())
                else:
                    self.meshtext.set_text('<No Mesh in window!>')
            else:
                self.meshtext.set_text('---')
            if newmesh is not self.currentMesh:
                self.currentMesh = newmesh
                self.updateData()
            if newmesh is not None:
                self.meshwidget.set_value(newmesh.path())
            else:
                self.meshwidget.set_value(None)

    def updatePosition(self, position):
        debug.mainthreadTest()
        if not self.freezeflag:
            if position != self.position:
                self.position = position
                self.updateData()
            if position is not None:
                self.xtext.set_text("%-13.6g" % position.x)
                self.ytext.set_text("%-13.6g" % position.y)
            else:
                # Probably not required -- position is initially None,
                # but having once been set, probably can't become
                # None.
                self.xtext.set_text("")
                self.ytext.set_text("")
                

    def updateData(self):
        if not self.freezeflag:
            self.databox.set_sensitive(1)
            if self.datawidget:
                self.datawidget.destroy()
            if self.currentMesh is not None and self.position is not None:
                op = self.outputwidget.get_value()
                if op is not None and not op.incomputable(self.currentMesh):
                    element = self.currentMesh.enclosingElement(self.position)
                    if element is not None:
                        masterpos = element.to_master(self.position.x,
                                                      self.position.y)
                        val = op.evaluate(self.currentMesh.getObject(),
                                          [element], [[masterpos]])[0]
                        self.datawidget = outputvalwidgets.makeWidget(val)
                        self.databox.pack_start(self.datawidget.gtk,
                                                expand=1, fill=1, padding=3)
                        self.datawidget.show()
        else:                           # frozen
            self.databox.set_sensitive(0)


        

###############################
        
def openMeshData(gfxwindow, position, output=None):
    MeshDataGUI(gfxwindow, position, output)

