# -*- python -*-
# $RCSfile: rank3tensorwidgets.py,v $
# $Revision: 1.17 $
# $Author: reida $
# $Date: 2005/10/18 18:22:08 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Contains size-specific descendants of SymmetricMatrixInput,
# which have as values objects of the SymmMatrix type
# from SWIG.engine.SymmMatrix.


from oof2.SWIG.engine import fieldindex
from oof2.SWIG.engine import rank3tensor
from oof2.common import debug
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import matrixparamwidgets

MatrixInput = matrixparamwidgets.MatrixInput

########################################

class C1Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        
            
        for i in range(3):
            for j in range(6):
                self.floats[(i, j)].gtk.connect(
                    "activate", self.new_value, None)
                self.floats[(i, j)].gtk.connect(
                    "focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0, 0))
            self.floats[(0,1)].set_value(self.value.get(0,1, 1))
            self.floats[(0,2)].set_value(self.value.get(0,2, 2))
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(0,4)].set_value(self.value.get(0,0, 2)*2.0)
            self.floats[(0,5)].set_value(self.value.get(0,0, 1)*2.0)
            ##
            self.floats[(1,0)].set_value(self.value.get(1,0, 0))
            self.floats[(1,1)].set_value(self.value.get(1,1, 1))
            self.floats[(1,2)].set_value(self.value.get(1,2, 2))
            self.floats[(1,3)].set_value(self.value.get(1, 2, 1)*2.0)
            self.floats[(1,4)].set_value(self.value.get(1,0, 2)*2.0)
            self.floats[(1,5)].set_value(self.value.get(1,0, 1)*2.0)
            ##
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,1, 1))
            self.floats[(2,2)].set_value(self.value.get(2,2, 2))
            self.floats[(2,3)].set_value(self.value.get(2, 2, 1)*2.0)
            self.floats[(2,4)].set_value(self.value.get(2,0, 2)*2.0)
            self.floats[(2,5)].set_value(self.value.get(2,0, 1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C1Rank3Tensor(
                d11= self.floats[(0,0)].get_value(),
                d12 = self.floats[(0,1)].get_value(),
                d13 = self.floats[(0,2)].get_value(),
                d14 = self.floats[(0,3)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d16 = self.floats[(0,5)].get_value(),
                d21 = self.floats[(1,0)].get_value(),
                d22 = self.floats[(1,1)].get_value(),
                d23 = self.floats[(1,2)].get_value(),
                d24 = self.floats[(1,3)].get_value(),
                d25 = self.floats[(1,4)].get_value(),
                d26 = self.floats[(1,5)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d32 = self.floats[(2,1)].get_value(),
                d33 = self.floats[(2,2)].get_value(),
                d34 = self.floats[(2,3)].get_value(),
                d35 = self.floats[(2,4)].get_value(),
                d36 = self.floats[(2,5)].get_value()
                )
        finally:
            self.set_values(result)

def _C1Rank3Tensor_makeWidget(self, scope=None):
    return C1Rank3TensorWidget(self, scope)

rank3tensor.C1Rank3TensorParameter.makeWidget = _C1Rank3Tensor_makeWidget


########################################

class C2Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3), (0,5), (1,0), (1,1), (1,2), (1,4), (2,3), (2,5)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(0,5)].set_value(self.value.get(0,0, 1)*2.0)
            ##
            self.floats[(1,0)].set_value(self.value.get(1,0, 0))
            self.floats[(1,1)].set_value(self.value.get(1,1, 1))
            self.floats[(1,2)].set_value(self.value.get(1,2, 2))
            self.floats[(1,4)].set_value(self.value.get(1,0, 2)*2.0)
            ##
            self.floats[(2,3)].set_value(self.value.get(2, 2, 1)*2.0)
            self.floats[(2,5)].set_value(self.value.get(2,0, 1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C2Rank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d16 = self.floats[(0,5)].get_value(),
                d21 = self.floats[(1,0)].get_value(),
                d22 = self.floats[(1,1)].get_value(),
                d23 = self.floats[(1,2)].get_value(),
                d25 = self.floats[(1,4)].get_value(),
                d34 = self.floats[(2,3)].get_value(),
                d36 = self.floats[(2,5)].get_value()
                )
        finally:
            self.set_values(result)

def _C2Rank3Tensor_makeWidget(self, scope=None):
    return C2Rank3TensorWidget(self, scope)

rank3tensor.C2Rank3TensorParameter.makeWidget = _C2Rank3Tensor_makeWidget


########################################

class CsRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        excluded = [(0,0), (0,1), (0,2), (0,4), (1,3),
                    (1,5), (2,0), (2,1), (2,2), (2,4)]
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0, 0))
            self.floats[(0,1)].set_value(self.value.get(0,1, 1))
            self.floats[(0,2)].set_value(self.value.get(0,2, 2))
            self.floats[(0,4)].set_value(self.value.get(0,0, 2)*2.0)
            ##
            self.floats[(1,3)].set_value(self.value.get(1, 2, 1)*2.0)
            self.floats[(1,5)].set_value(self.value.get(1,0, 1)*2.0)
            ##
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,1, 1))
            self.floats[(2,2)].set_value(self.value.get(2,2, 2))
            self.floats[(2,4)].set_value(self.value.get(2,0, 2)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value
    
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.CsRank3Tensor(
                d11= self.floats[(0,0)].get_value(),
                d12 = self.floats[(0,1)].get_value(),
                d13 = self.floats[(0,2)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d24 = self.floats[(1,3)].get_value(),
                d26 = self.floats[(1,5)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d32 = self.floats[(2,1)].get_value(),
                d33 = self.floats[(2,2)].get_value(),
                d35 = self.floats[(2,4)].get_value()
                )
        finally:
            self.set_values(result)

def _CsRank3Tensor_makeWidget(self, scope=None):
    return CsRank3TensorWidget(self, scope)

rank3tensor.CsRank3TensorParameter.makeWidget = _CsRank3Tensor_makeWidget


########################################

class D2Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        excluded = [(0,3), (1,4), (2,5)]
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)

        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(1,4)].set_value(self.value.get(1,0, 2)*2.0)
            self.floats[(2,5)].set_value(self.value.get(2,0, 1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D2Rank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d25 = self.floats[(1,4)].get_value(),
                d36 = self.floats[(2,5)].get_value()
                )
        finally:
            self.set_values(result)

def _D2Rank3Tensor_makeWidget(self, scope=None):
    return D2Rank3TensorWidget(self, scope)

rank3tensor.D2Rank3TensorParameter.makeWidget = _D2Rank3Tensor_makeWidget

########################################

class C2vRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        excluded = [(0,4), (1,3), (2,0), (2,1), (2,2)]
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,4)].set_value(self.value.get(0,0, 2)*2.0)
            self.floats[(1,3)].set_value(self.value.get(1, 2, 1)*2.0)
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,1, 1))
            self.floats[(2,2)].set_value(self.value.get(2,2, 2))
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C2vRank3Tensor(
                d15 = self.floats[(0,4)].get_value(),
                d24 = self.floats[(1,3)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d32 = self.floats[(2,1)].get_value(),
                d33 = self.floats[(2,2)].get_value()
                )
        finally:
            self.set_values(result)

def _C2vRank3Tensor_makeWidget(self, scope=None):
    return C2vRank3TensorWidget(self, scope)

rank3tensor.C2vRank3TensorParameter.makeWidget = _C2vRank3Tensor_makeWidget

########################################

class C4Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3), (0,4), (2,0), (2,2)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)

        self.set_values(param.value)
        
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C4Rank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value() )
        finally:
            self.set_values(result)
            
    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(1,4)].set_value(-self.value.get(0,2,1)*2.0)
            ##
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,0, 0))
            ##
            self.floats[(0,4)].set_value(self.value.get(0,0, 2)*2.0)
            self.floats[(1,3)].set_value(self.value.get(0,0, 2)*2.0)
            ##
            self.floats[(2,2)].set_value(self.value.get(2,2, 2))

        self.gtk.show_all()
    def get_value(self):
        return self.value
  

def _C4Rank3Tensor_makeWidget(self, scope=None):
    return C4Rank3TensorWidget(self, scope)

rank3tensor.C4Rank3TensorParameter.makeWidget = _C4Rank3Tensor_makeWidget

########################################

class C4iRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3), (0,4), (2,0), (2,5)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        self.set_values(param.value)
        ##
        
    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(1,4)].set_value(self.value.get(0,2,1)*2.0)
            ##
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,0, 0))
            ##
            self.floats[(0,4)].set_value(self.value.get(0,0, 2)*2.0)
            self.floats[(1,3)].set_value(-self.value.get(0,0, 2)*2.0)
            ##
            self.floats[(2,5)].set_value(self.value.get(2,0, 1))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C4iRank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d36 = self.floats[(2,5)].get_value()
                )
        finally:
            self.set_values(result)

def _C4iRank3Tensor_makeWidget(self, scope=None):
    return C4iRank3TensorWidget(self, scope)

rank3tensor.C4iRank3TensorParameter.makeWidget = _C4iRank3Tensor_makeWidget


########################################

class D4Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(1,4)].set_value(-self.value.get(0,2,1)*2.0)
            ##
        self.gtk.show_all()

    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D4Rank3Tensor(
                d14 = self.floats[(0,3)].get_value()
                )
        finally:
            self.set_values(result)

def _D4Rank3Tensor_makeWidget(self, scope=None):
    return D4Rank3TensorWidget(self, scope)

rank3tensor.D4Rank3TensorParameter.makeWidget = _D4Rank3Tensor_makeWidget


########################################

class C4vRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(2,0), (2,2), (0,4)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)


    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,0, 0))
            self.floats[(2,2)].set_value(self.value.get(2,2,2))
            self.floats[(0,4)].set_value(self.value.get(0,0,2)*2.0)
            self.floats[(1,3)].set_value(self.value.get(1,1,2)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C4vRank3Tensor(
                d15 = self.floats[(0,4)].get_value(),
                d31= self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value()     )
        finally:
            self.set_values(result)

def _C4vRank3Tensor_makeWidget(self, scope=None):
    return C4vRank3TensorWidget(self, scope)

rank3tensor.C4vRank3TensorParameter.makeWidget = _C4vRank3Tensor_makeWidget


########################################

class D2dRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3), (2,5)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)


    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0)
            self.floats[(1,4)].set_value(self.value.get(0,2,1)*2.0)
            ##
            self.floats[(2,5)].set_value(self.value.get(2,0, 1))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D2dRank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d36 = self.floats[(2,5)].get_value()
                )
        finally:
            self.set_values(result)

def _D2dRank3Tensor_makeWidget(self, scope=None):
    return D2dRank3TensorWidget(self, scope)

rank3tensor.D2dRank3TensorParameter.makeWidget = _D2dRank3Tensor_makeWidget

########################################

class C3Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,0), (0,3), (0,4), (1,1), (2,0), (2,2)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0, 0, 0)) ## d11
            self.floats[(0,1)].set_value(-self.value.get(0, 0, 0)) ## -d11
            self.floats[(1,5)].set_value(-2*self.value.get(0, 0, 0)) ## 2*d11
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0) ## d14
            self.floats[(1,4)].set_value(-self.value.get(0, 2, 1)*2.0) ## -d14
            self.floats[(0,4)].set_value(self.value.get(0, 0, 2)*2.0) ## d15
            self.floats[(1,3)].set_value(self.value.get(0, 0, 2)*2.0) ## d15
            self.floats[(1,1)].set_value(self.value.get(1,1,1)) ## d22
            self.floats[(1,0)].set_value(-self.value.get(1,1,1)) ## -d22
            self.floats[(0,5)].set_value(-2.0*self.value.get(1,1,1)) ## -2*d22
            self.floats[(2,0)].set_value(self.value.get(2,0,0)) ## d31
            self.floats[(2,1)].set_value(self.value.get(2,0,0)) ## d31
            self.floats[(2,2)].set_value(self.value.get(2,2,2)) ## d33

        self.gtk.show_all()
    def get_value(self):
        return self.value
    
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C3Rank3Tensor(
                d11 = self.floats[(0,0)].get_value(),
                d14 = self.floats[(0,3)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d22 = self.floats[(1,1)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value()
                )
        finally:
            self.set_values(result)

def _C3Rank3Tensor_makeWidget(self, scope=None):
    return C3Rank3TensorWidget(self, scope)

rank3tensor.C3Rank3TensorParameter.makeWidget = _C3Rank3Tensor_makeWidget




########################################

class D3Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,0), (0,3)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0, 0, 0)) ## d11
            self.floats[(0,1)].set_value(-self.value.get(0, 0, 0)) ## -d11
            self.floats[(1,5)].set_value(-2*self.value.get(0, 0, 0)) ## 2*d11
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0) ## d14
            self.floats[(1,4)].set_value(-self.value.get(0, 2, 1)*2.0) ## -d14
            

        self.gtk.show_all()
    def get_value(self):
        return self.value
    
    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D3Rank3Tensor(
                d11 = self.floats[(0,0)].get_value(),
                d14 = self.floats[(0,3)].get_value()
                )
        finally:
            self.set_values(result)

def _D3Rank3Tensor_makeWidget(self, scope=None):
    return D3Rank3TensorWidget(self, scope)

rank3tensor.D3Rank3TensorParameter.makeWidget = _D3Rank3Tensor_makeWidget



########################################

class C3vRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3), (1,1), (2,0), (2,2)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0, 2, 1)*2.0) ## d14
            self.floats[(1,4)].set_value(-self.value.get(0, 2, 1)*2.0) ## -d14
            self.floats[(1,1)].set_value(self.value.get(1,1,1)) ## d22
            self.floats[(1,0)].set_value(-self.value.get(1,1,1)) ## -d22
            self.floats[(0,5)].set_value(-2.0*self.value.get(1,1,1)) ## -2*d22
            self.floats[(2,0)].set_value(self.value.get(2,0,0)) ## d31
            self.floats[(2,1)].set_value(self.value.get(2,0,0)) ## d31
            self.floats[(2,2)].set_value(self.value.get(2,2,2)) ## d33

        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C3vRank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d22 = self.floats[(1,1)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value()
                )
        finally:
            self.set_values(result)

def _C3vRank3Tensor_makeWidget(self, scope=None):
    return C3vRank3TensorWidget(self, scope)

rank3tensor.C3vRank3TensorParameter.makeWidget = _C3vRank3Tensor_makeWidget


########################################

class C6Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(2,0), (2,2), (0,4), (0,3)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)

    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,0, 0))
            self.floats[(2,2)].set_value(self.value.get(2,2,2))
            self.floats[(0,4)].set_value(self.value.get(0,0,2)*2.0)
            self.floats[(1,3)].set_value(self.value.get(1,1,2)*2.0)
            self.floats[(0,3)].set_value(self.value.get(0,2,1)*2.0)
            self.floats[(1,4)].set_value(-self.value.get(0,2,1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C6Rank3Tensor(
                d14 = self.floats[(0,3)].get_value(),
                d15 = self.floats[(0,4)].get_value(),
                d31 = self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value()
                )
        finally:
            self.set_values(result)

def _C6Rank3Tensor_makeWidget(self, scope=None):
    return C6Rank3TensorWidget(self, scope)

rank3tensor.C6Rank3TensorParameter.makeWidget = _C6Rank3Tensor_makeWidget



########################################

class D6iRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,0), (1,1)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0, 0, 0)) ## d11
            self.floats[(0,1)].set_value(-self.value.get(0, 0, 0)) ## -d11
            self.floats[(1,5)].set_value(-2*self.value.get(0, 0, 0)) ## 2*d11
            
            self.floats[(1,1)].set_value(self.value.get(1,1,1)) ## d22
            self.floats[(1,0)].set_value(-self.value.get(1,1,1)) ## -d22
            self.floats[(0,5)].set_value(-2.0*self.value.get(1,1,1)) ## -2*d22
            

        self.gtk.show_all()

    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D6iRank3Tensor(
                d11 = self.floats[(0,0)].get_value(),
                d22 = self.floats[(1,1)].get_value()
                )
        finally:
            self.set_values(result)

def _D6iRank3Tensor_makeWidget(self, scope=None):
    return D6iRank3TensorWidget(self, scope)

rank3tensor.D6iRank3TensorParameter.makeWidget = _D6iRank3Tensor_makeWidget




########################################

class D6Rank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            
            self.floats[(0,3)].set_value(self.value.get(0,2,1)*2.0)
            self.floats[(1,4)].set_value(-self.value.get(0,2,1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D6Rank3Tensor(
                d14 = self.floats[(0,3)].get_value()
                )
        finally:
            self.set_values(result)

def _D6Rank3Tensor_makeWidget(self, scope=None):
    return D6Rank3TensorWidget(self, scope)

rank3tensor.D6Rank3TensorParameter.makeWidget = _D6Rank3Tensor_makeWidget




########################################

class C6vRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(2,0), (2,2), (0,4)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        ##
        self.set_values(param.value)

    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(2,0)].set_value(self.value.get(2,0, 0))
            self.floats[(2,1)].set_value(self.value.get(2,0, 0))
            self.floats[(2,2)].set_value(self.value.get(2,2,2))
            self.floats[(0,4)].set_value(self.value.get(0,0,2)*2.0)
            self.floats[(1,3)].set_value(self.value.get(1,1,2)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.C6vRank3Tensor(
                d15 = self.floats[(0,4)].get_value(),
                d31= self.floats[(2,0)].get_value(),
                d33 = self.floats[(2,2)].get_value()   )
        finally:
            self.set_values(result)

def _C6vRank3Tensor_makeWidget(self, scope=None):
    return C6vRank3TensorWidget(self, scope)

rank3tensor.C6vRank3TensorParameter.makeWidget = _C6vRank3Tensor_makeWidget



########################################

class D3hRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(1,1)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)

    def set_values(self, value): 
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(1,1)].set_value(self.value.get(1,1,1)) ## d22
            self.floats[(1,0)].set_value(-self.value.get(1,1,1)) ## -d22
            self.floats[(0,5)].set_value(-2.0*self.value.get(1,1,1)) ## -2*d22
            

        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.D3hRank3Tensor(
                d22 = self.floats[(1,1)].get_value()
                )
        finally:
            self.set_values(result)

def _D3hRank3Tensor_makeWidget(self, scope=None):
    return D3hRank3TensorWidget(self, scope)

rank3tensor.D3hRank3TensorParameter.makeWidget = _D3hRank3Tensor_makeWidget


########################################

class TdRank3TensorWidget(MatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        excluded = [(0,3)]
        MatrixInput.__init__(self,"",3,6,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k not in excluded :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                f.gtk.connect("activate", self.new_value, None)
                f.gtk.connect("focus_out_event", self.new_value)
        
        self.set_values(param.value)
    
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,3)].set_value(self.value.get(0,2,1)*2.0)
            self.floats[(1,4)].set_value(self.value.get(0,2,1)*2.0)
            self.floats[(2,5)].set_value(self.value.get(0,2,1)*2.0)
        self.gtk.show_all()
    def get_value(self):
        return self.value

    def new_value(self, gtk, event):
        result = self.value
        try:
            result = rank3tensor.TdRank3Tensor(
                d14 = self.floats[(0,3)].get_value()
                )
        finally:
            self.set_values(result)

def _TdRank3Tensor_makeWidget(self, scope=None):
    return TdRank3TensorWidget(self, scope)

rank3tensor.TdRank3TensorParameter.makeWidget = _TdRank3Tensor_makeWidget


