// -*- C++ -*-
// $RCSfile: node.h,v $
// $Revision: 1.12 $
// $Author: reida $
// $Date: 2006/01/25 21:59:51 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

class Node;

#ifndef NODE_H
#define NODE_H

#include "common/coord.h"
#include "field.h"
#include "freedom.h"
#include "common/pythonexportable.h"
#include <vector>
#include <iostream>

class Equation;
class FEMesh;
class NodalEquation;
class ElementMapNodeIterator;

class Node : public PythonExportable<Node>, public Position {
private:
  static const std::string classname_;
  static const std::string modulename_;
protected:
  Node(int n, const Coord &p);	// only called by FEMesh::newMapNode()
  Node(const Node&);		// prohibited
  Node &operator=(const Node&);	// prohibited

  // bool selected;
  Coord pos;
  int index_;	// unique for this Node, but not necessarily constant in time.
  void set_index(int idx) { index_ = idx; } // called by FEMesh
public:
  int index() const { return index_; }
  
  virtual ~Node() {}
  virtual const std::string &classname() const { return classname_; }
  virtual const std::string &modulename() const { return modulename_; }
  virtual Coord position() const { return pos; } // get original position

  /*
  int isselected() const { return selected; }
  void select() { selected = 1; }
  void unselect() { selected = 0; }
  void toggleselect() { if(selected) unselect(); else select(); }
  */
  int hashfunc(const Coord&, double&);

  virtual int ndof() const { return 0; }
  virtual int neqn() const { return 0; }

  friend bool operator==(const Node&, const Node&);
  friend bool operator!=(const Node &n0, const Node &n1) { return !(n0==n1); }

  friend std::ostream& operator<<(std::ostream&, const Node&);
  friend class FEMesh;

  virtual std::string repr() const;
  virtual std::string ctor() const;
};

// Node at which function parameters (degrees of freedom, nodal
// equations) are defined

class FuncNode : public Node {
private:
  static const std::string classname_;
protected:
  FuncNode(FEMesh*, int n, const Coord &p);
                  // only called by FEMesh::newFuncNode()
public:
  virtual ~FuncNode() {}
  virtual const std::string &classname() const { return classname_; }
  // Used by the regression tests to check solutions.
  Coord displaced_position(const FEMesh*) const;
  
  virtual int ndof() const { return doflist.size(); }
  virtual int neqn() const { return eqnlist.size(); }

  bool fixed() const; // is any Field fixed at this node?

  void AddField(FEMesh*, const Field&);	// allocate and initialize dof
  void RemoveField(FEMesh*, const Field&);

  void AddEquation(FEMesh*, const Equation&);
  void RemoveEquation(FEMesh*, const Equation&);

  // Lists of all the degrees of freedom and nodal equations defined
  // at this Node.  doflist is public because the Fields need access
  // to it, and eqnlist is public because Equation needs accesss to it.
  std::vector<DegreeOfFreedom*> doflist;
  std::vector<NodalEquation*> eqnlist;
  friend class FEMesh;
  
  virtual std::string repr() const;
  virtual std::string ctor() const;
};

#endif
