// -*- C++ -*-
// $RCSfile: heatconductivity.h,v $
// $Revision: 1.21 $
// $Author: langer $
// $Date: 2005/12/02 19:32:45 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

// heat conductivity property

#ifndef HEATCONDUCTIVITY_H
#define HEATCONDUCTIVITY_H


#include "engine/property.h"
#include "engine/matvec.h"
#include "engine/symmmatrix.h"
#include <string>

class Element;
class Material;
class FEMesh;
class OrientationProp;
class ScalarField;
class VectorFlux;
class ElementNodeIterator;

class HeatConductivityProp : public Property {
public:
  HeatConductivityProp(PyObject *registry, const std::string &name);
  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*,
			  const MasterPosition&) const;

  virtual void cross_reference(Material*) = 0;
  virtual int K_order(const FEMesh*, const Element*) const;
  virtual bool is_symmetric(const FEMesh*) const { return true; }
protected:
  MATRIX_D conductivitytensor;
  ScalarField *temperature;
  VectorFlux *heat_flux;
};

class IsoHeatConductivityProp : public HeatConductivityProp {
public:
  IsoHeatConductivityProp(PyObject *registry, 
		      const std::string &name, double kappa);
  virtual void cross_reference(Material*) {}
  virtual void precompute();
private:
  double kappa_;
};

class AnisoHeatConductivityProp : public HeatConductivityProp {
public:
  AnisoHeatConductivityProp(PyObject *registry,
			    const std::string &name,
			    SymmMatrix3 *kappa);
  virtual void cross_reference(Material*); // finds Orientation
  virtual void precompute();
private:
  SymmMatrix3 kappa_;
  OrientationProp *orientation;
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// distributed heat source

class HeatSourceProp : public Property {
private:
  double qdot_; // external heat dumped in per unit area per unit time.
  VectorFlux *heat_flux;
public:
  HeatSourceProp(PyObject *registry, const std::string &name, double qd);
  double qdot() { return qdot_; }
  virtual int K_order(const Element*) const;
  virtual void fluxrhs(const FEMesh*, const Element*, 
		       const Flux*, FluxData*, const MasterPosition&) const;
   virtual bool is_symmetric(const FEMesh* mesh) const { return true; }
};

#endif
